@@ -1,14 +1,13 @@
 package App::Sets;
-{
-  $App::Sets::VERSION = '0.973';
-}
+$App::Sets::VERSION = '0.974';
 
-# ABSTRACT: set operations in Perl
 
 use strict;
 use warnings;
+
+# ABSTRACT: set operations in Perl
+
 use English qw( -no_match_vars );
-use IPC::Open2 qw< open2 >;
 use 5.010;
 use Getopt::Long
   qw< GetOptionsFromArray :config pass_through no_ignore_case bundling >;
@@ -16,8 +15,11 @@ use Pod::Usage qw< pod2usage >;
 use Log::Log4perl::Tiny qw< :easy :dead_if_first LOGLEVEL >;
 use App::Sets::Parser;
 use App::Sets::Iterator;
+use App::Sets::Operations;
+use App::Sets::Sort qw< sort_filehandle >;
 
 my %config = (
+   binmode => ':raw:encoding(UTF-8)',
    loglevel => 'INFO',
    parsedebug => 0,
 );
@@ -32,18 +34,27 @@ sub populate_config {
       if exists $ENV{SETS_LOGLEVEL};
    $config{parsedebug}  = $ENV{SETS_PARSEDEBUG}
       if exists $ENV{SETS_PARSEDEBUG};
+   $config{internal_sort} = $ENV{SETS_INTERNAL_SORT}
+      if exists $ENV{SETS_INTERNAL_SORT};
+   $config{binmode} = $ENV{SETS_BINMODE} if $ENV{SETS_BINMODE};
    GetOptionsFromArray(
       \@args, \%config, qw< man help usage version
-        trim|t! sorted|s! cache|cache-sorted|S=s
+        binmode|b=s
+        cache|cache-sorted|S=s
+        internal_sort|internal-sort|I!
         loglevel|l=s
+        sorted|s!
+        trim|t!
         >
      )
      or pod2usage(
       -verbose  => 99,
       -sections => 'USAGE',
      );
-   our $VERSION; ${VERSION} //= '0.972' unless defined $VERSION;
-   pod2usage(message => "$0 $VERSION", -verbose => 99, -sections => ' ')
+   $App::Sets::VERSION
+        //= '0.972' unless defined $App::Sets::VERSION;
+   pod2usage(message => "$0 $App::Sets::VERSION", -verbose => 99,
+       -sections => ' ')
      if $config{version};
    pod2usage(
       -verbose  => 99,
@@ -105,10 +116,12 @@ sub run {
    my $expression = App::Sets::Parser::parse($input, 0);
    LOGLEVEL($config{loglevel});
 
+   binmode STDOUT, $config{binmode};
+
    my $it = expression($expression);
    while (defined(my $item = $it->drop())) {
-      print $item;
-      print "\n" if $config{trim};
+      print {*STDOUT} $item;
+      print {*STDOUT} "\n" if $config{trim};
    }
    return;
 } ## end sub run
@@ -123,20 +136,14 @@ sub expression {
    my ($expression) = @_;
    if (ref $expression) {    # operation
       my ($op, $l, $r) = @$expression;
-      return __PACKAGE__->can($op)->(expression($l), expression($r));
+      my $sub = App::Sets::Operations->can($op);
+      return $sub->(expression($l), expression($r));
    }
    else {                    # plain file
       return file($expression);
    }
 } ## end sub expression
 
-sub _sort_filehandle {
-   my ($filename) = @_;
-   open my $fh, '-|', 'sort', '-u', $filename
-     or LOGDIE "open() sort -u '$filename': $OS_ERROR";
-   return $fh;
-} ## end sub _sort_filehandle
-
 sub file {
    my ($filename) = @_;
    LOGDIE "invalid file '$filename'\n"
@@ -147,7 +154,7 @@ sub file {
       if (!-e $cache_filename) {    # generate cache file
          WARN "generating cached sorted file "
            . "'$cache_filename', might wait a bit...";
-         my $ifh = _sort_filehandle($filename);
+         my $ifh = sort_filehandle($filename, \%config);
          open my $ofh, '>', $cache_filename
            or LOGDIE "open('$cache_filename') for output: $OS_ERROR";
          while (<$ifh>) {
@@ -169,7 +176,7 @@ sub file {
    } ## end if ($config{sorted})
    else {
       INFO "opening '$filename' and sorting on the fly";
-      $fh = _sort_filehandle($filename);
+      $fh = sort_filehandle($filename, \%config);
    }
    return App::Sets::Iterator->new(
       sub {
@@ -182,83 +189,9 @@ sub file {
    );
 } ## end sub file
 
-sub intersect {
-   my ($l, $r) = @_;
-   my ($lh, $rh);
-   return App::Sets::Iterator->new(
-      sub {
-         while ('necessary') {
-            $lh //= $l->drop() // last;
-            $rh //= $r->drop() // last;
-            if ($lh eq $rh) {
-               my $retval = $lh;
-               $lh = $rh = undef;
-               return $retval;
-            }
-            elsif ($lh gt $rh) {
-               $rh = undef;
-            }
-            else {
-               $lh = undef;
-            }
-         } ## end while ('necessary')
-         return undef;
-      }
-   );
-} ## end sub intersect
-
-sub union {
-   my ($l, $r) = @_;
-   my ($lh, $rh);
-   return App::Sets::Iterator->new(
-      sub {
-         while (defined($lh = $l->head()) && defined($rh = $r->head())) {
-            if ($lh eq $rh) {
-               $r->drop();
-               return $l->drop();
-            }
-            elsif ($lh lt $rh) {
-               return $l->drop();
-            }
-            else {
-               return $r->drop();
-            }
-         } ## end while (defined($lh = $l->head...
-         while (defined($lh = $l->drop())) {
-            return $lh;
-         }
-         while (defined($rh = $r->drop())) {
-            return $rh;
-         }
-         return undef;
-      }
-   );
-} ## end sub union
-
-sub minus {
-   my ($l, $r) = @_;
-   my ($lh, $rh);
-   return App::Sets::Iterator->new(
-      sub {
-         while (defined($lh = $l->head()) && defined($rh = $r->head())) {
-            if ($lh eq $rh) {    # shared, drop both
-               $r->drop();
-               $l->drop();
-            }
-            elsif ($lh lt $rh) {    # only in left, OK!
-               return $l->drop();
-            }
-            else {                  # only in right, go on
-               $r->drop();
-            }
-         } ## end while (defined($lh = $l->head...
-         return $l->drop();
-      }
-   );
-} ## end sub minus
-
 1;
 
+__END__
 
 =pod
 
@@ -268,7 +201,7 @@ App::Sets - set operations in Perl
 
 =head1 VERSION
 
-version 0.973
+version 0.974
 
 =head1 AUTHOR
 
@@ -276,7 +209,7 @@ Flavio Poletti <polettix@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011-2013 by Flavio Poletti polettix@cpan.org.
+Copyright (C) 2011-2016 by Flavio Poletti polettix@cpan.org.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
@@ -286,6 +219,3 @@ but without any warranty; without even the implied warranty of
 merchantability or fitness for a particular purpose.
 
 =cut
-
-
-__END__
